@@ -15,7 +15,7 @@
  #
  # Originally developed by Gregory Trubetskoy.
  #
- # $Id: Session.py 378945 2006-02-19 19:51:17Z jgallacher $
+ # $Id: Session.py 416523 2006-06-23 00:25:29Z jgallacher $
 
 import apache, Cookie
 import _apache
@@ -56,11 +56,11 @@ def _init_rnd():
     g = random.Random(time.time())
     result = [g]
     for i in range(gennum - 1):
-       laststate = g.getstate()
-       g = random.Random()
-       g.setstate(laststate)
-       g.jumpahead(1000000)
-       result.append(g)
+        laststate = g.getstate()
+        g = random.Random()
+        g.setstate(laststate)
+        g.jumpahead(1000000)
+        result.append(g)
 
     return result
 
@@ -342,7 +342,7 @@ class DbmSession(BaseSession):
             self._dbmtype = __import__(module)
         
     def _get_dbm(self):
-        result = self._dbmtype.open(self._dbmfile, 'c')
+        result = self._dbmtype.open(self._dbmfile, 'c', stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP)
         if self._dbmtype is anydbm:
             self._set_dbm_type()
         return result
