@@ -15,7 +15,7 @@
  #
  # This file originally written by Sterling Hughes
  #
- # $Id: psp.py 102649 2004-02-16 19:47:28Z grisha $
+ # $Id: psp.py 374268 2006-02-02 05:31:45Z nlehuen $
 
 import apache, Session, util, _psp
 import _apache
@@ -111,12 +111,14 @@ class PSP:
             self.load_from_file()
         else:
 
-            cached = strcache.get(string)
+            cached = mem_scache.get(string)
             if cached:
                 self.code = cached
             else:
-                self.code = _psp.parsestring(string)
-                strcache.store(string)
+                source = _psp.parsestring(string)
+                code = compile(source, "__psp__", "exec")
+                mem_scache.store(string,code)
+                self.code = code
 
     def cache_get(self, filename, mtime):
 
@@ -188,7 +190,10 @@ class PSP:
         # does this code use session?
         session = None
         if "session" in code.co_names:
-            session = Session.Session(req)
+            if hasattr(req, 'session'):
+                session = req.session
+            else:
+                session = Session.Session(req) 
 
         # does this code use form?
         form = None
@@ -315,21 +320,38 @@ def dbm_cache_type(dbmfile):
         return anydbm
 
 def dbm_cache_store(srv, dbmfile, filename, mtime, val):
-    
+   
     dbm_type = dbm_cache_type(dbmfile)
-    _apache._global_lock(srv, "pspcache")
+
+    # NOTE: acquiring a lock for the dbm file (also applies to dbm_cache_get)
+    # See http://issues.apache.org/jira/browse/MODPYTHON-69
+    # In mod_python versions < 3.2 "pspcache" was used as the lock key.
+    # ie.  _apache._global_lock(srv, key, index)
+    # Assuming there are 32 mutexes (the default in 3.1.x), "pspcache"
+    # will hash to one of  31 mutexes (index 0 is reserved). Therefore
+    # there is a 1 in 31 chance for a hash collision if a session is
+    # used in the same request, which would result in a deadlock. This
+    # has been confirmed by testing.
+    # We can avoid this by using index 0 and setting the key to None.
+    # Lock index 0 is also used by DbmSession for locking it's dbm file, 
+    # but since the lock is not held for the duration of the request there
+    # should not be any additional deadlock issues. Likewise, the lock
+    # here is only held for a short time, so it will not interfere
+    # with DbmSession file locking.
+  
+    _apache._global_lock(srv, None, 0)
     try:
         dbm = dbm_type.open(dbmfile, 'c')
         dbm[filename] = "%d %s" % (mtime, code2str(val))
     finally:
         try: dbm.close()
         except: pass
-        _apache._global_unlock(srv, "pspcache")
+        _apache._global_unlock(srv, None, 0)
 
 def dbm_cache_get(srv, dbmfile, filename, mtime):
 
     dbm_type = dbm_cache_type(dbmfile)
-    _apache._global_lock(srv, "pspcache")
+    _apache._global_lock(srv, None, 0)
     try:
         dbm = dbm_type.open(dbmfile, 'c')
         try:
@@ -342,7 +364,7 @@ def dbm_cache_get(srv, dbmfile, filename, mtime):
     finally:
         try: dbm.close()
         except: pass
-        _apache._global_unlock(srv, "pspcache")
+        _apache._global_unlock(srv, None, 0)
 
 
 class HitsCache:
@@ -358,8 +380,8 @@ class HitsCache:
 
     def get(self, key):
         if self.cache.has_key(key):
-            hist, val = self.cache[key]
-            self.cache[key] = (hits+1, code)
+            hits, val = self.cache[key]
+            self.cache[key] = (hits+1, val)
             return val
         else:
             return None
