@@ -18,7 +18,7 @@
  *
  * _apachemodule.c 
  *
- * $Id: _apachemodule.c 332807 2005-11-12 18:11:14Z nlehuen $
+ * $Id: _apachemodule.c 420297 2006-07-09 13:53:06Z nlehuen $
  *
  */
 
@@ -315,8 +315,13 @@ static PyObject *parse_qsl(PyObject *self, PyObject *args)
             _PyString_Resize(&key, strlen(ckey));
             _PyString_Resize(&val, strlen(cval));
 
-            if (key && val)
-                PyList_Append(pairs, Py_BuildValue("(O,O)", key, val));
+            if (key && val) {
+                PyObject* listitem = Py_BuildValue("(O,O)", key, val);
+                if(listitem) {
+                    PyList_Append(pairs, listitem);
+                    Py_DECREF(listitem);
+                }
+            }
 
         }
         Py_XDECREF(pair);
