@@ -18,7 +18,7 @@
  *
  * mod_python.c 
  *
- * $Id: mod_python.c 367835 2006-01-10 23:36:37Z jgallacher $
+ * $Id: mod_python.c 397813 2006-04-28 09:06:55Z grahamd $
  *
  * See accompanying documentation and source code comments 
  * for details.
@@ -80,7 +80,7 @@ static PyInterpreterState *make_interpreter(const char *name, server_rec *srv)
 static PyObject * make_obcallback(char *name, server_rec *s)
 {
 
-    PyObject *m;
+    PyObject *m = NULL;
     PyObject *obCallBack = NULL;
 
     /* This makes _apache appear imported, and subsequent
@@ -97,22 +97,93 @@ static PyObject * make_obcallback(char *name, server_rec *s)
      */
 
     if (! ((m = PyImport_ImportModule(MODULENAME)))) {
+        PyObject *path;
+
         ap_log_error(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, 0, s,
-                     "make_obcallback: could not import %s.\n", (!MODULENAME) ? "<null>" : MODULENAME);
+                     "make_obcallback: could not import %s.\n",
+                     (!MODULENAME) ? "<null>" : MODULENAME);
+
         PyErr_Print();
         fflush(stderr); 
+
+        path = PyObject_Repr(PySys_GetObject("path"));
+
+        ap_log_error(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, 0, s,
+                     "make_obcallback: Python path being used \"%s\".",
+                     PyString_AsString(path));
+
+        Py_DECREF(path);
+
+        return NULL;
     }
     
-    if (m && ! ((obCallBack = PyObject_CallMethod(m, INITFUNC, "sO", name, MpServer_FromServer(s))))) {
+    if (m && ! ((obCallBack = PyObject_CallMethod(m,
+                 INITFUNC, "sO", name, MpServer_FromServer(s))))) {
+
+        const char *mp_compile_version = MPV_STRING;
+        const char *mp_dynamic_version = "<unknown>";
+        PyObject *o = NULL;
+        PyObject *d = NULL;
+        PyObject *f = NULL;
+
         ap_log_error(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, 0, s,
-                     "make_obcallback: could not call %s.\n", (!INITFUNC) ? "<null>" : INITFUNC);
+                     "make_obcallback: could not call %s.\n",
+                     (!INITFUNC) ? "<null>" : INITFUNC);
+
         PyErr_Print();
         fflush(stderr); 
+
+        m = PyImport_ImportModule("mod_python");
+        if (m) {
+            d = PyModule_GetDict(m);
+            o = PyDict_GetItemString(d, "version");
+            f = PyDict_GetItemString(d, "__file__");
+
+            if (o) {
+                mp_dynamic_version = PyString_AsString(o);
+            }
+        }
+
+        if (strcmp(mp_compile_version, mp_dynamic_version) != 0) {
+            ap_log_error(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, 0, s,
+                         "make_obcallback: mod_python version mismatch, expected '%s', found '%s'.",
+                         mp_compile_version, mp_dynamic_version);
+            ap_log_error(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, 0, s,
+                         "make_obcallback: mod_python modules location '%s'.",
+                         PyString_AsString(f));
+        }
+
+        Py_XDECREF(o);
+        Py_XDECREF(f);
     }
+
+      Py_XDECREF(m);
     
     return obCallBack;
 }
 
+/**
+ ** save_interpreter
+ **
+ *      Save away the interpreter.
+ */
+
+static interpreterdata *save_interpreter(const char *name, PyInterpreterState *istate)
+{
+    PyObject *p;
+    interpreterdata *idata = NULL;
+
+    idata = (interpreterdata *)malloc(sizeof(interpreterdata));
+    idata->istate = istate;
+    /* obcallback will be created on first use */
+    idata->obcallback = NULL; 
+    p = PyCObject_FromVoidPtr((void *) idata, NULL);
+    PyDict_SetItemString(interpreters, (char *)name, p);
+    Py_DECREF(p);
+
+    return idata;
+}
+
 /**
  ** get_interpreter
  **
@@ -146,15 +217,9 @@ static interpreterdata *get_interpreter(const char *name, server_rec *srv)
     if (!p)
     {
         PyInterpreterState *istate = make_interpreter(name, srv);
-        if (istate) {
-            idata = (interpreterdata *)malloc(sizeof(interpreterdata));
-            idata->istate = istate;
-            /* obcallback will be created on first use */
-            idata->obcallback = NULL; 
-            p = PyCObject_FromVoidPtr((void *) idata, NULL);
-            PyDict_SetItemString(interpreters, (char *)name, p);
-            Py_DECREF(p);
-        }
+
+        if (istate)
+            idata = save_interpreter(name, istate);
     }
     else {
         idata = (interpreterdata *)PyCObject_AsVoidPtr(p);
@@ -314,8 +379,14 @@ static apr_status_t init_mutexes(server_rec *s, apr_pool_t *p, py_global_config
     int max_clients;
     int locks;
     int n;
+    char *val;
+    char *mutex_dir;
+    py_config *conf;
+    
+    conf = (py_config *) ap_get_module_config(s->module_config, 
+                                           &python_module);
 
-    /* figre out maximum possible concurrent connections */
+    /* figure out maximum possible concurrent connections */
     /* MAX_DAEMON_USED seems to account for MaxClients, as opposed to
        MAX_DAEMONS, which is ServerLimit
     */
@@ -336,19 +407,31 @@ static apr_status_t init_mutexes(server_rec *s, apr_pool_t *p, py_global_config
     max_clients = (((max_threads <= 0) ? 1 : max_threads) *
                    ((max_procs <= 0) ? 1 : max_procs));
 
-    /* XXX On some systems the locking mechanism chosen uses valuable
+    /* On some systems the locking mechanism chosen uses valuable
        system resources, notably on RH 8 it will use sysv ipc for
        which Linux by default provides only 128 semaphores
        system-wide, and on many other systems flock is used, which
-       results in a relatively large number of open files. So for now
-       we get by with MAX_LOCKS constant for lack of a better
-       solution.
+       results in a relatively large number of open files.
+
+       The maximum number of locks can be specified at
+       compile time using "./configure --with-max-locks value" or
+       at run time with "PythonOption mod_python.mutex_locks value".
+
+       If the PythonOption directive is used, it must be in a 
+       server config context, otherwise it will be ignored.
 
        The optimal number of necessary locks is not clear, perhaps a
        small number is more than sufficient - if someone took the
        time to run some research on this, that'd be most welcome!
     */
-    locks = (max_clients > MAX_LOCKS) ? MAX_LOCKS : max_clients; 
+    val = apr_table_get(conf->options, "mod_python.mutex_locks");
+    if (val) {
+        locks = atoi(val);
+    } else {
+        locks = MAX_LOCKS;
+    }
+
+    locks = (max_clients > locks) ? locks : max_clients; 
 
     ap_log_error(APLOG_MARK, APLOG_NOTICE, 0, s,
                  "mod_python: Creating %d session mutexes based "
@@ -360,14 +443,35 @@ static apr_status_t init_mutexes(server_rec *s, apr_pool_t *p, py_global_config
     glb->nlocks = locks;
     glb->parent_pid = getpid();
 
+#if !defined(OS2) && !defined(WIN32) && !defined(BEOS) && !defined(NETWARE)
+    /* On some sytems a directory for the mutex lock files is required.
+       This mutex directory can be specifed at compile time using
+       "./configure --with-mutex-dir value" or at run time with
+       "PythonOption mod_python.mutex_directory value".
+     
+       If the PythonOption directive is used, it must be in a 
+       server config context, otherwise it will be ignored.
+
+       XXX Should we check if mutex_dir exists and has the correct
+       permissions?
+    */
+    mutex_dir = apr_table_get(conf->options, "mod_python.mutex_directory");
+    if (!mutex_dir)
+        mutex_dir = MUTEX_DIR;
+    
+    ap_log_error(APLOG_MARK, APLOG_NOTICE, 0, s,
+                 "mod_python: using mutex_directory %s ",
+                 mutex_dir);
+#endif
+
     for (n=0; n<locks; n++) {
         apr_status_t rc;
         apr_global_mutex_t **mutex = glb->g_locks;
 
 #if !defined(OS2) && !defined(WIN32) && !defined(BEOS) && !defined(NETWARE)
         char fname[255];
-
-        snprintf(fname, 255, "/tmp/mpmtx%d%d", glb->parent_pid, n);
+        /* XXX What happens if len(mutex_dir) > 255 - len(mpmtx%d%d)? */   
+        snprintf(fname, 255, "%s/mpmtx%d%d", mutex_dir, glb->parent_pid, n);
 #else
         char *fname = NULL;
 #endif
@@ -378,7 +482,7 @@ static apr_status_t init_mutexes(server_rec *s, apr_pool_t *p, py_global_config
                          "mod_python: Failed to create global mutex %d of %d (%s).",
                          n, locks, (!fname) ? "<null>" : fname);
             if (n > 1) {
-                /* we were able to crate at least two, so lets just print a 
+                /* we were able to create at least two, so lets just print a 
                    warning/hint and proceed
                 */
                 ap_log_error(APLOG_MARK, APLOG_ERR, 0, s,
@@ -424,13 +528,27 @@ static apr_status_t reinit_mutexes(server_rec *s, apr_pool_t *p, py_global_confi
 {
     int n;
 
+#if !defined(OS2) && !defined(WIN32) && !defined(BEOS) && !defined(NETWARE)
+    /* Determine the directory to use for mutex lock files. 
+       See init_mutexes function for more details.
+    */
+    char *mutex_dir;
+    py_config *conf;
+
+    conf = (py_config *) ap_get_module_config(s->module_config, 
+                                           &python_module);
+    mutex_dir = apr_table_get(conf->options, "mod_python.mutex_directory");
+    if (!mutex_dir)
+        mutex_dir = MUTEX_DIR;
+#endif
+
     for (n=0; n< glb->nlocks; n++) {
         apr_status_t rc;
         apr_global_mutex_t **mutex = glb->g_locks;
 
 #if !defined(OS2) && !defined(WIN32) && !defined(BEOS) && !defined(NETWARE)
         char fname[255];
-        snprintf(fname, 255, "/tmp/mpmtx%d%d", glb->parent_pid, n);
+        snprintf(fname, 255, "%s/mpmtx%d%d", mutex_dir, glb->parent_pid, n);
 #else
         char *fname = NULL;
 #endif
@@ -492,7 +610,22 @@ static int python_init(apr_pool_t *p, apr_pool_t *ptemp,
     const char *userdata_key = "python_init";
     apr_status_t rc;
 
-    /* fudge for Mac OS X with Apache 1.3 where Py_IsInitialized() broke */
+    const char *py_compile_version = PY_VERSION;
+    const char *py_dynamic_version = 0;
+
+    /* The "initialized" flag is a fudge for Mac OS X. It
+     * addresses two issues. The first is that when an Apache
+     * "restart" is performed, Apache will unload the mod_python
+     * shared object, but not the Python framework. This means
+     * that when "python_init()" is called after the restart,
+     * the mod_python initialization will not run if only the
+     * initialized state of Python is checked, because Python
+     * is already initialized. The second problem is that for
+     * some older revisions of Mac OS X, even on the initial
+     * startup of Apache, the "Py_IsInitialized()" function
+     * would return true and mod_python wouldn't initialize
+     * itself correctly and would crash.
+     */
     static int initialized = 0;
 
     apr_pool_userdata_get(&data, userdata_key, s->process->pool);
@@ -504,9 +637,23 @@ static int python_init(apr_pool_t *p, apr_pool_t *ptemp,
 
     /* mod_python version */
     ap_add_version_component(p, VERSION_COMPONENT);
-    
+
+    py_dynamic_version = strtok((char *)Py_GetVersion(), " ");
+
+    if (strcmp(py_compile_version, py_dynamic_version) != 0) {
+        ap_log_error(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, 0, s,
+                     "python_init: Python version mismatch, expected '%s', found '%s'.",
+                     py_compile_version, py_dynamic_version);
+        ap_log_error(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, 0, s,
+                     "python_init: Python executable found '%s'.",
+                     Py_GetProgramFullPath());
+        ap_log_error(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, 0, s,
+                     "python_init: Python path being used '%s'.",
+                     Py_GetPath());
+    }
+
     /* Python version */
-    sprintf(buff, "Python/%.200s", strtok((char *)Py_GetVersion(), " "));
+    sprintf(buff, "Python/%.200s", py_dynamic_version);
     ap_add_version_component(p, buff);
 
     /* global config */
@@ -530,9 +677,6 @@ static int python_init(apr_pool_t *p, apr_pool_t *ptemp,
         /* create and acquire the interpreter lock */
         PyEval_InitThreads();
 #endif
-        /* Release the thread state because we will never use 
-         * the main interpreter, only sub interpreters created later. */
-        PyThreadState_Swap(NULL); 
 
         /* create the obCallBack dictionary */
         interpreters = PyDict_New();
@@ -541,12 +685,11 @@ static int python_init(apr_pool_t *p, apr_pool_t *ptemp,
                          "python_init: PyDict_New() failed! No more memory?");
             exit(1);
         }
-        
+
 #ifdef WITH_THREAD
         /* release the lock; now other threads can run */
         PyEval_ReleaseLock();
 #endif
-
     }
     return OK;
 }
@@ -1609,9 +1752,11 @@ static const char *directive_PythonOption(cmd_parms *cmd, void * mconfig,
     if(val!=NULL) {
         apr_table_set(conf->options, key, val);
 
-        conf = ap_get_module_config(cmd->server->module_config,
-                                    &python_module);
-        apr_table_set(conf->options, key, val);
+        if (!cmd->path) {
+            conf = ap_get_module_config(cmd->server->module_config,
+                                        &python_module);
+            apr_table_set(conf->options, key, val);
+        }
     }
     else {
     	/** We don't remove the value, but set it
@@ -1620,9 +1765,11 @@ static const char *directive_PythonOption(cmd_parms *cmd, void * mconfig,
     	    an entry string precisely means 'remove the value' */
         apr_table_set(conf->options, key, "");
 
-        conf = ap_get_module_config(cmd->server->module_config,
-                                    &python_module);
-        apr_table_set(conf->options, key, "");
+        if (!cmd->path) {
+            conf = ap_get_module_config(cmd->server->module_config,
+                                        &python_module);
+            apr_table_set(conf->options, key, "");
+        }
     }
 
     return NULL;
@@ -1840,6 +1987,11 @@ static void PythonChildInitHandler(apr_pool_t *p, server_rec *s)
     PyEval_AcquireLock();
 #endif
     PyOS_AfterFork();
+
+    save_interpreter(MAIN_INTERPRETER, PyThreadState_Get()->interp);
+
+    PyThreadState_Swap(NULL);
+
 #ifdef WITH_THREAD
     PyEval_ReleaseLock();
 #endif
