@@ -18,7 +18,7 @@
  *
  * serverobject.c 
  *
- * $Id: serverobject.c 102649 2004-02-16 19:47:28Z grisha $
+ * $Id: serverobject.c 393438 2006-04-12 11:16:42Z grahamd $
  *
  */
 
@@ -66,6 +66,21 @@ static PyObject * server_get_config(serverobject *self)
     return MpTable_FromTable(conf->directives);
 }
 
+/**
+ ** server.get_options(server self)
+ **
+ *     Returns the options set through PythonOption directives.
+ *     unlike req.get_options, this one returns the per-server config
+ */
+
+static PyObject * server_get_options(serverobject *self)
+{
+    py_config *conf =
+        (py_config *) ap_get_module_config(self->server->module_config,
+                                           &python_module);
+    return MpTable_FromTable(conf->options);
+}
+
 /**
  ** server.register_cleanup(req, handler, data)
  **
@@ -101,7 +116,7 @@ static PyObject *server_register_cleanup(serverobject *self, PyObject *args)
     ci->server_rec = self->server;
     Py_INCREF(handler);
     ci->handler = handler;
-    ci->interpreter = req->interpreter;
+    ci->interpreter = strdup(req->interpreter);
     if (data) {
         Py_INCREF(data);
         ci->data = data;
@@ -120,6 +135,7 @@ static PyObject *server_register_cleanup(serverobject *self, PyObject *args)
 
 static PyMethodDef server_methods[] = {
     {"get_config",           (PyCFunction) server_get_config,        METH_NOARGS},
+    {"get_options",          (PyCFunction) server_get_options,       METH_NOARGS},
     {"register_cleanup",     (PyCFunction) server_register_cleanup,  METH_VARARGS},
     { NULL, NULL } /* sentinel */
 };
@@ -206,7 +222,7 @@ static PyObject *getsrv_recmbr_ah(serverobject *self, void *name)
 {
     const PyMemberDef *md = find_memberdef(server_rec_mbrs, name);
     apr_array_header_t *ah = 
-        (apr_array_header_t *)((char *)self->server + md->offset);
+        *(apr_array_header_t **)((char *)self->server + md->offset);
 
     return tuple_from_array_header(ah);
 }
