@@ -18,7 +18,7 @@
  *
  * util.c 
  *
- * $Id: util.c 357352 2005-12-17 17:31:35Z jgallacher $
+ * $Id: util.c 420297 2006-07-09 13:53:06Z nlehuen $
  *
  * See accompanying documentation and source code comments 
  * for details.
@@ -396,6 +396,8 @@ PyObject *cfgtree_walk(ap_directive_t *dir)
 
         PyList_Append(list, t);
 
+        Py_DECREF(t);
+
         if (dir->first_child) {
 
             PyObject *child = cfgtree_walk(dir->first_child);
@@ -403,6 +405,8 @@ PyObject *cfgtree_walk(ap_directive_t *dir)
                 return PyErr_NoMemory();
 
             PyList_Append(list, child);
+            
+            Py_DECREF(child);
 
         }
 
