@@ -52,11 +52,14 @@
  # information on the Apache Software Foundation, please see
  # <http://www.apache.org/>.
  #
- # $Id: tests.py 102748 2004-02-23 17:37:16Z grisha $
+ # $Id: tests.py 393776 2006-04-13 09:48:19Z grahamd $
  #
 
 # mod_python tests
 
+from __future__ import generators
+from mod_python.python22 import *
+
 from mod_python import apache
 import unittest
 import re
@@ -64,6 +67,10 @@ import time
 import os
 import cStringIO
 
+# This is used for mod_python.publisher security tests
+_SECRET_PASSWORD = 'root'
+__ANSWER = 42
+
 class SimpleTestCase(unittest.TestCase):
 
     def __init__(self, methodName, req):
@@ -211,8 +218,8 @@ class SimpleTestCase(unittest.TestCase):
             self.fail("req.allowed_xmethods should be an empty tuple")
             
         log("    req.allowed_methods: %s" % `req.allowed_methods`)
-        if req.allowed_methods:
-            self.fail("req.allowed_methods should be None")
+        if req.allowed_methods != ():
+            self.fail("req.allowed_methods should be an empty tuple")
             
         log("    req.sent_bodyct: %s" % `req.sent_bodyct`)
         if req.sent_bodyct != 0:
@@ -307,6 +314,10 @@ class SimpleTestCase(unittest.TestCase):
         log("    req.content_encoding: %s" % `req.content_encoding`)
         if req.content_encoding:
             self.fail("req.content_encoding should be None")
+             
+        log("    req.content_languages: %s" % `req.content_languages`)
+        if req.content_languages != ():
+            self.fail("req.content_languages should be an empty tuple")
             
         log("    req.vlist_validator: %s" % `req.vlist_validator`)
         if req.vlist_validator:
@@ -379,7 +390,7 @@ class SimpleTestCase(unittest.TestCase):
 
         log("req.get_options(): %s" % `req.get_options()`)
         if req.get_options() != apache.table({"testing":"123"}):
-            self.fail("get_options() should contain 'testing':'123'")
+            self.fail("get_options() should contain 'testing':'123', contains %s"%req.get_options().items())
 
     def test_req_get_remote_host(self):
 
@@ -468,6 +479,15 @@ class SimpleTestCase(unittest.TestCase):
         log("    server.limit_req_fields: %s" % `server.limit_req_fields`)
         if server.limit_req_fields != 100:
             self.fail("server.limit_req_fields should be 100")
+  
+        log("    server.names: %s" % `server.names`)
+        if server.names != ():
+            self.fail("server.names should be an empty tuple")           
+ 
+        log("    server.wild_names: %s" % `server.wild_names`)
+        if server.wild_names != ():
+            self.fail("server.wild_names should be an empty tuple")
+            
 
     def test_connection_members(self):
 
@@ -577,6 +597,36 @@ def simple_handler(req):
         
     return apache.OK
 
+def req_add_bad_handler(req):
+    # bad_handler does not exist so adding it should 
+    # should raise an AttributeError exception
+    
+    req.log_error("req_add_bad_handler")
+    req.add_handler("PythonHandler", "tests::bad_handler")
+    req.write("test ok")
+
+    return apache.OK
+
+def req_add_empty_handler_string(req):
+    # Adding an empty string as a handler should should 
+    # should raise an exception
+    
+    req.log_error("req_add_empty_handler_string")
+    req.add_handler("PythonHandler", "")
+    req.write("no exception")
+
+    return apache.OK
+
+def accesshandler_add_handler_to_empty_hl(req):
+    # Prior to version 3.2.6, adding a python handler 
+    # to and empty handler list would cause a segfault
+ 
+    req.secret_message = "foo"
+    req.log_error("accesshandler_add_handler_to_empty_hl")
+    req.add_handler("PythonHandler", "tests::simple_handler")
+
+    return apache.OK
+
 def req_allow_methods(req):
 
     req.allow_methods(["PYTHONIZE"])
@@ -594,13 +644,12 @@ def req_get_basic_auth_pw(req):
 
 def req_requires(req):
 
-    req.user = "blah" # or else!
-
     if req.requires() == ('valid-user',):
         req.write("test ok")
         return apache.DONE
 
-    return apache.OK
+    req.write("test failed")
+    return apache.DONE
 
 def req_document_root(req):
 
@@ -678,9 +727,67 @@ def req_sendfile(req):
 
     req.sendfile(fname, 2, 7)
 
+    # os.remove(fname)
+    return apache.OK
+
+def req_sendfile2(req):
+
+    import tempfile
+    fname  = tempfile.mktemp("txt")
+    f = open(fname, "w")
+    f.write("0123456789"*100);
+    f.close()
+
+    req.sendfile(fname)
+
+    # os.remove(fname)
+    return apache.OK
+ 
+def req_sendfile3(req):
+    """Check if sendfile handles symlinks properly.
+       This is only valid on posix systems.
+    """
+
+    import tempfile
+    # note mktemp is deprecated in python 2.3. Should use mkstemp instead.
+    fname  = tempfile.mktemp("txt")
+    f = open(fname, "w")
+    f.write("0123456789"*100);
+    f.close()
+    fname_symlink =  '%s.lnk' % fname
+    os.symlink(fname, fname_symlink)
+    req.sendfile(fname_symlink)
+    os.remove(fname_symlink)
     os.remove(fname)
     return apache.OK
 
+def req_server_get_options(req):
+
+    try:
+        server_options = req.server.get_options()
+        assert(server_options["global"] == "1")
+        assert(server_options["override"] == "1")
+
+        request_options = req.get_options()
+        assert(request_options["global"] == "1")
+        assert(request_options["override"] == "2")
+        assert(request_options["local"] == "1")
+    except:
+        req.write('test failed')
+    else:
+        req.write('test ok')
+
+    return apache.OK
+
+def fileupload(req):
+    from mod_python import util
+    import md5
+    fields = util.FieldStorage(req)
+    f = fields.getfirst('testfile')
+    
+    req.write(md5.new(f.file.read()).hexdigest())
+    return apache.OK
+
 def srv_register_cleanup(req):
 
     req.cleanup_data = "test ok"
@@ -689,6 +796,21 @@ def srv_register_cleanup(req):
 
     return apache.OK
 
+def apache_register_cleanup(req):
+
+    req.cleanup_data = "test 2 ok"
+    apache.register_cleanup(cleanup, req)
+    req.write("registered server cleanup that will write to log")
+
+    return apache.OK
+
+def apache_exists_config_define(req):
+    if apache.exists_config_define('FOOBAR'):
+        req.write('FOOBAR')
+    else:
+        req.write('NO_FOOBAR')
+    return apache.OK
+
 def util_fieldstorage(req):
 
     from mod_python import util
@@ -813,7 +935,80 @@ def Session_Session(req):
         req.write("test ok")
 
     return apache.OK
+
+def test_sys_argv(req):
+    import sys
+    req.write(repr(sys.argv))
+    return apache.OK
         
+def PythonOption_items(req):
+    options = req.get_options().items()
+    
+    # The tests may using PythonOption mod_python.* in the test configuration
+    # We need to remove those particular options so they don't interfer
+    # with this test result.
+    options = [ o for o in options if not o[0].startswith('mod_python') ]
+    
+    options.sort()
+    req.write(str(options))
+    return apache.OK
+
+def interpreter(req):
+    req.write(req.interpreter)
+    return apache.OK
+
+def index(req):
+    return "test ok, interpreter=%s" % req.interpreter
+
+def test_publisher(req):
+    return "test ok, interpreter=%s" % req.interpreter
+
+class OldStyleClassTest:
+    def __init__(self):
+        pass
+    def __call__(self, req):
+        return "test callable old-style instance ok"
+    def traverse(self, req):
+        return "test traversable old-style instance ok"
+old_instance = OldStyleClassTest()
+
+test_dict = {1:1, 2:2, 3:3}
+test_dict_keys = test_dict.keys
+
+def test_dict_iteration(req):
+    return test_dict_keys()
+    
+def test_generator(req):
+    c = 0
+    while c < 10:
+        yield c
+        c += 1
+
+class InstanceTest(object):
+    def __call__(self, req):
+        return "test callable instance ok"
+    def traverse(self, req):
+        return "test traversable instance ok"
+instance = InstanceTest()
+
+# Hierarchy traversal tests
+class Mapping(object):
+    def __init__(self,name):
+        self.name = name
+
+    def __call__(self,req):
+        return "Called %s"%self.name
+hierarchy_root = Mapping("root");
+hierarchy_root.page1 = Mapping("page1")
+hierarchy_root.page1.subpage1 = Mapping("subpage1")
+hierarchy_root.page2 = Mapping("page2")
+
+class Mapping2:
+    pass
+hierarchy_root_2 = Mapping2()
+hierarchy_root_2.__call__ = index
+hierarchy_root_2.page1 = index
+hierarchy_root_2.page2 = index
 
 def _test_table():
 
