@@ -13,7 +13,7 @@
  # implied.  See the License for the specific language governing
  # permissions and limitations under the License.
  #
- # $Id: test.py 378945 2006-02-19 19:51:17Z jgallacher $
+ # $Id: test.py 393776 2006-04-13 09:48:19Z grahamd $
  #
 
 """
@@ -179,6 +179,7 @@ LIBEXECDIR = testconf.LIBEXECDIR
 SERVER_ROOT = TESTHOME
 CONFIG = os.path.join(TESTHOME, "conf", "test.conf")
 DOCUMENT_ROOT = os.path.join(TESTHOME, "htdocs")
+TMP_DIR = os.path.join(TESTHOME, "tmp")
 PORT = 0 # this is set in fundUnusedPort()
  
     
@@ -220,6 +221,32 @@ def quoteIfSpace(s):
         s = '"%s"' % s
     return s
 
+def get_apache_version():
+
+    print "Checking Apache version...."
+    httpd = quoteIfSpace(HTTPD)
+    cmd = '%s -v' % (httpd)
+    (stdin,stdout) = os.popen2(cmd)
+
+    version_str = None
+    for line in stdout:
+        if line.startswith('Server version'):
+             version_str = line.strip()
+             break
+
+    if version_str:
+        version_str = version_str.split('/')[1]
+        major,minor,patch = version_str.split('.',3)
+        version = '%s.%s' % (major,minor)
+    else:
+        
+        print "Can't determine Apache version. Assuming 2.0"
+        version = '2.0'
+    print version
+    return version
+
+APACHE_VERSION = get_apache_version()
+
 class HttpdCtrl:
     # a mixin providing ways to control httpd
 
@@ -234,6 +261,10 @@ class HttpdCtrl:
             shutil.rmtree(logs)
         os.mkdir(logs)
 
+        if os.path.exists(TMP_DIR):
+            shutil.rmtree(TMP_DIR)
+        os.mkdir(TMP_DIR)
+
     def makeConfig(self, append=""):
 
         # create config files, etc
@@ -287,16 +318,29 @@ class HttpdCtrl:
             PidFile("logs/httpd.pid"),
             ServerName("127.0.0.1"),
             Listen(PORT),
+            PythonOption('mod_python.mutex_directory %s' % TMP_DIR),
             PythonOption('PythonOptionTest sample_value'),
             DocumentRoot(DOCUMENT_ROOT),
-            LoadModule("python_module %s" % quoteIfSpace(MOD_PYTHON_SO)),
-            IfModule("!mod_auth.c",
+            LoadModule("python_module %s" % quoteIfSpace(MOD_PYTHON_SO)))
+
+        if APACHE_VERSION == '2.2':
+            # mod_auth has been split into mod_auth_basic and some other modules
+            s.append(IfModule("!mod_auth_basic.c",
+                     LoadModule("auth_basic_module %s" %
+                                quoteIfSpace(os.path.join(modpath, "mod_auth_basic.so")))))
+
+            # Default KeepAliveTimeout is 5 for apache 2.2, but 15 in apache 2.0
+            # Explicitly set the value so it's the same as 2.0
+            s.append(KeepAliveTimeout("15"))
+        else:
+            s.append(IfModule("!mod_auth.c",
                      LoadModule("auth_module %s" %
                                 quoteIfSpace(os.path.join(modpath, "mod_auth.so")))))
 
+        s.append("\n# --APPENDED-- \n\n"+append)
+
         f = open(CONFIG, "w")
         f.write(str(s))
-        f.write("\n# --APPENDED-- \n\n"+append)
         f.close()
 
     def startHttpd(self,extra=''):
@@ -596,15 +640,16 @@ class PerRequestTestCase(unittest.TestCase):
     def test_req_requires_conf(self):
 
         c = VirtualHost("*",
-                        ServerName("test_req_requires"),
-                        DocumentRoot(DOCUMENT_ROOT),
-                        Directory(DOCUMENT_ROOT,
-                                  SetHandler("mod_python"),
-                                  AuthName("blah"),
-                                  AuthType("basic"),
-                                  Require("valid-user"),
-                                  PythonAuthenHandler("tests::req_requires"),
-                                  PythonDebug("On")))
+                    ServerName("test_req_requires"),
+                    DocumentRoot(DOCUMENT_ROOT),
+                    Directory(DOCUMENT_ROOT,
+                              SetHandler("mod_python"),
+                              AuthName("blah"),
+                              AuthType("dummy"),
+                              Require("valid-user"),
+                              PythonAuthenHandler("tests::req_requires"),
+                              PythonDebug("On")))
+
         return str(c)
 
     def test_req_requires(self):
@@ -877,6 +922,30 @@ class PerRequestTestCase(unittest.TestCase):
         else:
             print "\n  * Skipping req.sendfile() for a file which is a symbolic link"
 
+    def test_req_server_get_options_conf(self):
+
+        c = VirtualHost("*",
+                        ServerName("test_req_server_get_options"),
+                        DocumentRoot(DOCUMENT_ROOT),
+                        PythonDebug("Off"),
+                        PythonOption("global 1"),
+                        PythonOption("override 1"),
+                        Directory(DOCUMENT_ROOT,
+                                  SetHandler("mod_python"),
+                                  PythonHandler("tests::req_server_get_options"),
+                                  PythonOption("local 1"),
+                                  PythonOption("override 2"),
+                                  PythonDebug("On")))
+        return str(c)
+
+    def test_req_server_get_options(self):
+
+        print "\n  * Testing req.server.get_options()"
+
+        rsp = self.vhost_get("test_req_server_get_options")
+        if (rsp != "test ok"):
+            self.fail(`rsp`)
+
     def test_fileupload_conf(self):
 
         c = VirtualHost("*",
@@ -1352,6 +1421,7 @@ class PerRequestTestCase(unittest.TestCase):
                                   SetHandler("mod_python"),
                                   PythonHandler("tests"),
                                   PythonOption('PythonOptionTest ""'),
+                                  PythonOption('mod_python.mutex_directory ""'),
                                   PythonOption("testing 123"),
                                   PythonDebug("On")))
         return str(c)
@@ -1568,6 +1638,7 @@ class PerRequestTestCase(unittest.TestCase):
                         Directory(DOCUMENT_ROOT,
                                   SetHandler("mod_python"),
                                   PythonHandler("tests::Session_Session"),
+                                  PythonOption('session_directory "%s"' % TMP_DIR),
                                   PythonDebug("On")))
         return str(c)
 
@@ -1606,6 +1677,7 @@ class PerRequestTestCase(unittest.TestCase):
                         Directory(DOCUMENT_ROOT,
                                   SetHandler("mod_python"),
                                   PythonHandler("tests::Session_Session"),
+                                  PythonOption('session_directory "%s"' % TMP_DIR),
                                   PythonDebug("On")))
         return str(c)
 
@@ -1987,6 +2059,7 @@ class PerInstanceTestCase(unittest.TestCase, HttpdCtrl):
         perRequestSuite.addTest(PerRequestTestCase("test_req_sendfile"))
         perRequestSuite.addTest(PerRequestTestCase("test_req_sendfile2"))
         perRequestSuite.addTest(PerRequestTestCase("test_req_sendfile3"))
+        perRequestSuite.addTest(PerRequestTestCase("test_req_server_get_options"))
         perRequestSuite.addTest(PerRequestTestCase("test_fileupload"))
         perRequestSuite.addTest(PerRequestTestCase("test_fileupload_embedded_cr"))
         perRequestSuite.addTest(PerRequestTestCase("test_fileupload_split_boundary"))
